package ga.core;

import ga.core.GAIndividual;
import ga.core.GAProblem;
import ga.core.GAGene;
import ga.core.GAParams;
import ga.Evolve;

/**
 * Creates offspring from two parents.
 *
 * @author Olly Oechsle, University of Essex, Date: 21-May-2007
 * @version 1.0
 */
public class GABreeder {

    public static GAIndividual produceOffspring(GAProblem p, GAIndividual parent1, GAIndividual parent2, GAParams params) {

        // Create the child
        GAIndividual child = p.createIndividual();

        // work along the chromosone
        for (int i = 0; i < child.genes.length; i++) {
            GAGene gene = child.genes[i];

            // crossover
            if (Evolve.getRandomNumber() < params.getCrossoverProbability()) {

                // select a gene from one parent at random
                if (Evolve.getRandomNumber() > 0) {
                    gene.setValueFrom(parent1.genes[i]);
                } else {
                    gene.setValueFrom(parent2.genes[i]);
                }

                continue;

            }

            // mutation
            if (Evolve.getRandomNumber() < params.getMutationProbability()) {

                // reinitialise the gene with a random value
                if (Evolve.getRandomNumber() < 0.1) {
                    gene.setValue(gene.getInitialValue());
                } else {
                    gene.mutate();
                }

                continue;

            }


            // neither of the above, choose the average value
            double average = (parent1.genes[i].value + parent2.genes[i].value) / 2;

            gene.setValue(average);

        }

        return child;

    }

    public static GAIndividual[] onePointCrossover(GAProblem p, GAIndividual parent1, GAIndividual parent2) {

        // create the two children
        GAIndividual child1 = p.createIndividual();
        GAIndividual child2 = p.createIndividual();

        // choose the crossover point
        int crossoverPoint = (int) (Evolve.getRandomNumber() * child1.genes.length);

        // copy the genes over to both new children
        for (int i = 0; i < child1.genes.length; i++) {
            if (i < crossoverPoint) {
                child1.genes[i].setValueFrom(parent1.genes[i]);
                child2.genes[i].setValueFrom(parent2.genes[i]);
            } else {
                child1.genes[i].setValueFrom(parent2.genes[i]);
                child2.genes[i].setValueFrom(parent1.genes[i]);
            }
        }

        return new GAIndividual[]{child1, child2};

    }

    public static GAIndividual mutation(GAProblem p, GAIndividual parent)  {

        GAIndividual child1 = p.createIndividual();

        // choose the mutation probability - that about one gene will be changed by the mutation operator
        double mutationProbability = 1d / child1.genes.length;

        // copy the genes and make mutations as we go
        for (int i = 0; i < child1.genes.length; i++) {
            double mutation = Evolve.getRandomNumber();
            if (mutation <= mutationProbability) {
                // going to mutate the gene
                // choose a brand new value
                child1.genes[i].getInitialValue();
            } else {
                // just copy the value from the parent
                child1.genes[i].setValueFrom(parent.genes[i]);
            }
        }

        return child1;

    }

    public static GAIndividual reproduce(GAProblem p, GAIndividual parent)  {

        GAIndividual child1 = p.createIndividual();

        // copy the genes over to the child
        for (int i = 0; i < child1.genes.length; i++) {
            // just copy the value from the parent
            child1.genes[i].setValueFrom(parent.genes[i]);
        }

        return child1;

    }

}
