package ga.core;

import ga.Evolve;

/**
 * Parameters are tuned by the GA
 *
 * @author Olly Oechsle, University of Essex, Date: 18-May-2007
 * @version 1.0
 */
public abstract class GAGene {

    public double value;
    public double min;
    public double max;

    public GAGene(double min, double max) {
        this.min = min;
        this.max = max;
        this.value = getInitialValue();
    }

    public void mutate() {
        double amount = 0.20;
        double change = 1.0 - amount + (Evolve.getRandomNumber() * amount * 2);
        value *= change;
        if (value > max || value < min) value = getInitialValue();
    }

    public abstract double getInitialValue();

    public void setValueFrom(GAGene other) {
        this.value = other.value;
    }

    public void setValue(double value) {
        this.value = value;
    }

    public String toString() {
        return String.valueOf(value);
    }

}
