package ga.core;

import ga.Evolve;

/**
 * Contains settings for the GA algorithm
 *
 * @author Olly Oechsle, University of Essex, Date: 18-May-2007
 * @version 1.0
 */
public class GAParams {

    public static final int REPRODUCTION = 0;
    public static final int CROSSOVER = 1;
    public static final int MUTATION = 2;

    public static final int GENERATION_GAP_OVERLAP_OFF = 30;
    public static final int GENERATION_GAP_OVERLAP_ON = 31;

    protected int populationSize = 500;

    protected int generations = 50;

    protected int tournamentSize = 2;

    protected double crossoverProbability = 0.75;

    protected double mutationProbability = 0.2;

    protected int eliteCount = 0;

    protected int generationGapMethod = GENERATION_GAP_OVERLAP_ON;

    public int getGenerationGapMethod() {
        return generationGapMethod;
    }

    /**
     * The probability that one of the parents will be given, unchanged.
     */
    public double getCrossoverProbability() {
        return crossoverProbability;
    }

    /**
     * The probability that a newly initialised gene will be introduced.
     * @return
     */
    public double getMutationProbability() {
        return mutationProbability;
    }


    public void setCrossoverProbability(double crossoverProbability) {
        this.crossoverProbability = crossoverProbability;
    }

    public void setMutationProbability(double mutationProbability) {
        this.mutationProbability = mutationProbability;
    }

    public int getPopulationSize() {
        return populationSize;
    }

    public void setPopulationSize(int populationSize) {
        this.populationSize = populationSize;
    }

    public int getGenerations() {
        return generations;
    }

    public void setGenerations(int generations) {
        this.generations = generations;
    }

    public int getTournamentSize() {
        return tournamentSize;
    }

    public int getEliteCount() {
        return eliteCount;
    }

    /**
     * Returns a Genetic Operation, consisting of Reproduction, Crossover or Mutation, proportional
     * to the probabilities in the settings. If Crossover probability is set to 0.8, this method will return
     * crossover 80% of the time.
     */
    public int getOperator() {

        int mode = REPRODUCTION;

        // first decide which genetic operator to use
        double value = Evolve.getRandomNumber();
        if (value <= crossoverProbability) {
            // crossover mode
            mode = CROSSOVER;
        }

        if (value > crossoverProbability && value <= (crossoverProbability + mutationProbability)) {
            mode = MUTATION;
        }

        return mode;

    }
    
}
