package ga.listeners;

import ga.core.GAProblem;
import ga.core.GAIndividual;
import ga.util.GraphPanel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Provides a GUI for the GA
 *
 * @author Olly Oechsle, University of Essex, Date: 21-May-2007
 * @version 1.0
 */
public class GAGraphicalListener extends GAListener {

    protected JFrame window;
    protected JLabel generation, individual;
    protected GraphPanel graph;

    public void initialise(GAProblem p) {
        window = new JFrame();

        graph = new GraphPanel();
        window.add(graph, BorderLayout.CENTER);

        generation = new JLabel();
        individual = new JLabel();
        JPanel status = new JPanel(new FlowLayout(FlowLayout.LEFT));
        status.add(generation);
        status.add(individual);

        window.add(status, BorderLayout.SOUTH);

        window.setTitle(p.getName());
        window.setSize(500, 300);
        window.setLocation(100, 100);
        window.setVisible(true);

        window.addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
            
        });
    }

    public void onGenerationStart(int generation) {
        this.generation.setText(" Generation: " + generation);
    }

    public void onGenerationEnd(int generation, GAIndividual bestIndividual) {
        this.individual.setText(" Individual: " + bestIndividual.toString());
        graph.addValue(bestIndividual.getKozaFitness(), generation);
    }

    public void onEvolutionEnd(int generation, GAIndividual bestIndividual) {

        this.individual.setText(" Individual: " + bestIndividual.toString());
        graph.addValue(bestIndividual.getKozaFitness(), generation);        

        if (bestIndividual.getKozaFitness() == 0)  {
            alert("Found ideal individual.");
        } else {
            alert("Evolution stopped.");
        }
    }

    private void alert(String message) {
        JOptionPane.showMessageDialog(window, message);
    }

}
