package problems.jasmine;

import ga.core.GAIndividual;
import ga.core.GAProblemBoostable;
import ac.essex.ooechs.imaging.jasmine.JasmineProject;
import ac.essex.ooechs.imaging.jasmine.JasmineClass;
import ac.essex.ooechs.imaging.jasmine.JasmineUtils;
import ac.essex.ooechs.imaging.shapes.ExtraShapeData;
import ac.essex.ooechs.imaging.shapes.SegmentedShape;
import ac.essex.ooechs.adaboost.AdaBoostSample;

import java.util.Vector;
import java.io.File;

import problems.boosting.BoostingProblem;

/**
 * Solves a Jasmine Project
 *
 * @author Olly Oechsle, University of Essex, Date: 21-May-2007
 * @version 1.0
 */
public class ShapeProblem extends GAProblemBoostable {

    protected Vector<ExtraShapeData> trainingData;

    protected int classID;

    public static void main(String[] args) throws Exception {

        final String packageName = "pasta";
        final String projectName = "Pasta.jasmine";

        long start = System.currentTimeMillis();

        JasmineProject project = JasmineProject.load(new File("/home/ooechs/Desktop/JasmineProjects/" + projectName));

        Vector<JasmineClass> classes = project.getShapeClasses();

        StringBuffer buffer = new StringBuffer();

        for (int i = 0; i < classes.size(); i++) {

            JasmineClass jasmineClass = classes.elementAt(i);

            BoostingProblem p = new BoostingProblem(new ShapeProblem(project, jasmineClass.classID));

            p.boost(20);

            String className = "Detector_" + jasmineClass.name;

            p.save(className, "/home/ooechs/sxGA/src/problems/jasmine/solutions/" + packageName + "/");

            p.toJava(className);

            buffer.append("if (new " + className + "().classify(shape)) return " + jasmineClass.classID + ";\n");

            // reload the project
            project = JasmineProject.load(new File("/home/ooechs/Desktop/JasmineProjects/" + projectName));

        }

        System.out.println(buffer.toString());

        System.out.println("// Evolved " + classes.size() + " detectors in " + (System.currentTimeMillis() - start) + "ms");
                                             
    }

    public ShapeProblem(JasmineProject project, int classID) {

        this.classID = classID;

        // load the training data
        trainingData = JasmineUtils.getTrainingData(project);

        // make this a binary problem
        for (int i = 0; i < trainingData.size(); i++) {
            SegmentedShape segmentedShape = trainingData.elementAt(i).getShape();
            if (segmentedShape.classID != classID) segmentedShape.classID = 0;
            else segmentedShape.classID = 1;
        }

        System.out.println("Solving for class " + project.getShapeClass(classID).name);

    }


    public String getName() {
        return "GA Shape Problem";
    }


    public int countFunctions() {
        return 17;
    }

    public GAIndividual createIndividual() {
        return new ShapeIndividual(functionID);
    }

    public void evaluate(GAIndividual ind) {

        double e = 0;
        int hits = 0;

        for (int i = 0; i < trainingData.size(); i++) {

            ExtraShapeData shapeData = trainingData.elementAt(i);
            SegmentedShape shape = shapeData.getShape();

            int result = (int) ind.execute(new ShapeData(shape, shapeData));

            if (result != shape.classID) {
                e += weights == null? 1 : weights[i];
            } else {
                hits++;
            }

        }

        ind.setKozaFitness(e);
        ind.setHits(hits);
        
    }

    public AdaBoostSample[] getSamples() {
        AdaBoostSample[] samples = new AdaBoostSample[trainingData.size()];
        for (int i = 0; i < trainingData.size(); i++) {
            ExtraShapeData shape = trainingData.elementAt(i);
            samples[i] = new AdaBoostSample(shape, shape.getShape().classID);
        }
        return samples;
    }

}
